@@ -5,7 +5,7 @@
 /*                                                                            */
 /*  This file is part of NCrystal (see https://mctools.github.io/ncrystal/)   */
 /*                                                                            */
-/*  Copyright 2015-2021 NCrystal developers                                   */
+/*  Copyright 2015-2022 NCrystal developers                                   */
 /*                                                                            */
 /*  Licensed under the Apache License, Version 2.0 (the "License");           */
 /*  you may not use this file except in compliance with the License.          */
@@ -123,9 +123,7 @@ extern "C" {
                                                  ncrystal_scatter_t*,
                                                  ncrystal_absorption_t* );
 
-  /* Fine tuning factory availability and caching                                  */
-  NCRYSTAL_API void ncrystal_disable_caching(); /*NB: this concerns Info object caching only! */
-  NCRYSTAL_API void ncrystal_enable_caching();  /*NB: this concerns Info object caching only! */
+  /* Factory availablity:                                                          */
   NCRYSTAL_API int ncrystal_has_factory( const char * name );
 
   /*============================================================================== */
@@ -138,7 +136,7 @@ extern "C" {
   /*============================================================================== */
   /*============================================================================== */
 
-  /*Name of object:                                                                */
+  /*Name and UID of underlying ProcImpl::Process object:                           */
   NCRYSTAL_API const char * ncrystal_name(ncrystal_process_t);
 
   /*Determine if process is non-oriented (normally) or not (single-crystal):       */
@@ -184,14 +182,23 @@ extern "C" {
                                                double* alpha, double* beta, double* gamma,
                                                double* volume, unsigned* n_atoms );
 
-  /*Access various scalar information (return values of -1 means unavailable:      */
-  NCRYSTAL_API double ncrystal_info_gettemperature( ncrystal_info_t );
+  /*Access various scalar information:                                             */
   NCRYSTAL_API double ncrystal_info_getxsectabsorption( ncrystal_info_t );
   NCRYSTAL_API double ncrystal_info_getxsectfree( ncrystal_info_t );
   NCRYSTAL_API double ncrystal_info_getdensity( ncrystal_info_t );
   NCRYSTAL_API double ncrystal_info_getnumberdensity( ncrystal_info_t );
+  NCRYSTAL_API double ncrystal_info_getsld( ncrystal_info_t );
+  NCRYSTAL_API double ncrystal_info_gettemperature( ncrystal_info_t );/*-1 if N/A. */
+
+  /*State of matter (Unknown = 0, Solid = 1, Gas = 2, Liquid = 3)                  */
   NCRYSTAL_API int ncrystal_info_getstateofmatter( ncrystal_info_t );
 
+  /* Access phase information (nphases=0 means single phase)                       */
+  NCRYSTAL_API int ncrystal_info_nphases( ncrystal_info_t );
+  NCRYSTAL_API ncrystal_info_t ncrystal_info_getphase( ncrystal_info_t,
+                                                       int iphase,
+                                                       double* fraction );
+
   /*Access HKL info:                                                               */
   NCRYSTAL_API int ncrystal_info_nhkl( ncrystal_info_t ); /* -1 when not available */
   NCRYSTAL_API double ncrystal_info_hkl_dlower( ncrystal_info_t );
@@ -199,6 +206,12 @@ extern "C" {
   NCRYSTAL_API void ncrystal_info_gethkl( ncrystal_info_t, int idx,
                                           int* h, int* k, int* l, int* multiplicity,
                                           double * dspacing, double* fsquared );
+  /*All HKL indices in a given group (returns first value h[0]==k[0]==l[0]==0 if not possible). */
+  NCRYSTAL_API void ncrystal_info_gethkl_allindices( ncrystal_info_t, int idx,
+                                                     int* h, int* k, int* l );/* arrays of length multiplicity/2 */
+
+  NCRYSTAL_API double ncrystal_info_braggthreshold( ncrystal_info_t ); /* [Aa], -1 when not available */
+  NCRYSTAL_API int ncrystal_info_hklinfotype( ncrystal_info_t ); /* integer casted value of HKLInfoType */
 
   /*Access AtomInfo:                                                               */
   NCRYSTAL_API unsigned ncrystal_info_natominfo( ncrystal_info_t );/* 0=unavail    */
@@ -261,7 +274,10 @@ extern "C" {
   NCRYSTAL_API double ncrystal_info_dspacing_from_hkl( ncrystal_info_t, int h, int k, int l );
 
 
-  /* Composition (ncomponents=0 means composition unavailable):                    */
+  /* Composition (always >=1 component). Note that for multiphase objects, the     */
+  /* provided atomdataidx will be invalid, so it is important in general to use    */
+  /* ncrystal_create_component_atomdata(..) to access the atomdata object          */
+  /* associated with the i'th component, and NOT ncrystal_create_atomdata(..):     */
   NCRYSTAL_API unsigned ncrystal_info_ncomponents( ncrystal_info_t );
   NCRYSTAL_API void ncrystal_info_getcomponent( ncrystal_info_t, unsigned icomponent,
                                                 unsigned* atomdataindex,
@@ -275,6 +291,10 @@ extern "C" {
   NCRYSTAL_API ncrystal_atomdata_t ncrystal_create_atomdata( ncrystal_info_t,
                                                              unsigned atomdataindex );
 
+  /* Same, but via index in composition vector:                                    */
+  NCRYSTAL_API ncrystal_atomdata_t ncrystal_create_component_atomdata( ncrystal_info_t,
+                                                                       unsigned icomponent );
+
   /* Get atom data fields. Each object falls in one of three categories:           */
   /* 1) Natural elements (ncomponents=A=0,Z>0)                                     */
   /* 2) Single isotope (ncomponents=0, Z>0, A>=Z)                                  */
@@ -294,6 +314,28 @@ extern "C" {
                                                                      unsigned icomponent,
                                                                      double* fraction );
 
+  /* Get flattened breakdown of composition i.e. to set up a structureless base    */
+  /* material with same atoms in another application). Set prefernatelem to 1 or   */
+  /* 0 to indicate whether the breakdowns should prefer natural elements (A=0)     */
+  /* where possible, or always expand such to individial isotopes. The             */
+  /* natelemprovider argument is a callback to a function which provides such      */
+  /* natural abundances, and this is needed when prefernatelem==0 or when the      */
+  /* composition contains both natural elements and isotopes of a particular       */
+  /* element. If prefernatelem=1 the natelemprovider can be left out by passing    */
+  /* a NULL ptr, which will then still work for all materials except for this with */
+  /* the mentioned mixed compositions. Results are returned as a JSON string (list */
+  /* of (Z,A,frac) entries where A=0 indicates natural elements), which must be    */
+  /* cleaned up with ncrystal_dealloc_string).                                     */
+  /*                                                                               */
+  /* To keep the interface simple (if ugly), the natelemprovider function is       */
+  /* passed the Z value along with a preallocated int and double array of length   */
+  /* 128, which it must fill with corresponding A and abundance values. The return */
+  /* value is used to indicate the number of isotopes present. Thus, at most 128   */
+  /* isotopes can be returned (which should be plenty for all known elements).     */
+  /* It should simply return 0 in case of an element with no nat. abundance data.  */
+  char * ncrystal_get_flatcompos( ncrystal_info_t, int prefernatelem,
+                                  unsigned (*natelemprovider)(unsigned,unsigned*,double*) );
+
   /* Custom data section:                                                         */
   NCRYSTAL_API unsigned ncrystal_info_ncustomsections( ncrystal_info_t );
   NCRYSTAL_API const char* ncrystal_info_customsec_name( ncrystal_info_t, unsigned isection );
@@ -373,7 +415,7 @@ extern "C" {
   /*============================================================================== */
 
   /* Access TextData. Returns a string list of length 5:                           */
-  /* [contents, uid(as string), description, datatype, resolvedphyspath].          */
+  /* [contents, uid(as string), sourcename, datatype, resolvedphyspath].           */
   /* The last entry is optional and will be an empty str if absent.                */
   /* Must free list with call to ncrystal_dealloc_stringlist.                      */
   NCRYSTAL_API char** ncrystal_get_text_data( const char * name );
@@ -414,13 +456,31 @@ extern "C" {
   /*============================================================================== */
   /*============================================================================== */
 
-  /* Dump info to stdout:                                                          */
+  /* Dump info to stdout (1st fct is same as calling 2nd one with verbosity_lvl=0):*/
   NCRYSTAL_API void ncrystal_dump(ncrystal_info_t);
+  NCRYSTAL_API void ncrystal_dump_verbose(ncrystal_info_t, unsigned verbosity_lvl );
 
   /* Utility converting between neutron wavelength [Aa] to kinetic energy [eV]:    */
   NCRYSTAL_API double ncrystal_wl2ekin( double wl );
   NCRYSTAL_API double ncrystal_ekin2wl( double ekin );
 
+  /* Extract extra debug information about objects (as JSON string which must be   */
+  /* cleaned up with ncrystal_dealloc_string).                                     */
+  NCRYSTAL_API char * ncrystal_dbg_process( ncrystal_process_t );
+
+  /*UID of underlying Info or ProcImpl::Process object as string (must free with   */
+  /*call to ncrystal_dealloc_string:                                               */
+  NCRYSTAL_API char * ncrystal_process_uid(ncrystal_process_t);
+  NCRYSTAL_API char * ncrystal_info_uid(ncrystal_info_t);
+
+  /*Generate cfg-str variable documentation as string (must free with call to      */
+  /*ncrystal_dealloc_string). Mode 0 (full), 1 (short), 2 (json):                  */
+  NCRYSTAL_API char * ncrystal_gencfgstr_doc(int mode);
+
+  /*Underlying UID (in case density value or cfgdata  was overridden):             */
+  NCRYSTAL_API char * ncrystal_info_underlyinguid(ncrystal_info_t);
+
+
   /* Access internal DB for isotopes and natural elements.                         */
   /* NB: Will return invalid handle in case lookup failed. Otherwise, the          */
   /* returned object should eventually be unref'ed by calling code:                */
@@ -448,9 +508,12 @@ extern "C" {
                           double* temp_eff, double* origIntegral );
 
   /* Extract NCMatCfg variables which can not be inferred from an ncrystal_info_t  */
-  /* object and which might be needed in plugins (to be expanded as needed):       */
-  NCRYSTAL_API double ncrystal_decodecfg_packfact( const char * cfgstr );
+  /* object and which might be needed in plugins (to be expanded as needed).       */
+  /* Returned strings must be cleaned up with ncrystal_dealloc_string.             */
   NCRYSTAL_API unsigned ncrystal_decodecfg_vdoslux( const char * cfgstr );
+  NCRYSTAL_API char* ncrystal_decodecfg_json( const char * cfgstr );
+  /* Parse and reencode cfg (as NCrystal::MatCfg(cfgstr).toStrCfg()):              */
+  NCRYSTAL_API char* ncrystal_normalisecfg( const char * cfgstr );
 
   /* Clear various caches employed inside NCrystal:                                */
   NCRYSTAL_API void ncrystal_clear_caches();
@@ -480,14 +543,19 @@ extern "C" {
 #ifdef NCRYSTAL_VERSION_STR
 #  undef NCRYSTAL_VERSION_STR
 #endif
-#define NCRYSTAL_VERSION_MAJOR 2
-#define NCRYSTAL_VERSION_MINOR 7
-#define NCRYSTAL_VERSION_PATCH 3
-#define NCRYSTAL_VERSION   2007003 /* (1000000*MAJOR+1000*MINOR+PATCH)             */
-#define NCRYSTAL_VERSION_STR "2.7.3"
+#define NCRYSTAL_VERSION_MAJOR 3
+#define NCRYSTAL_VERSION_MINOR 4
+#define NCRYSTAL_VERSION_PATCH 1
+#define NCRYSTAL_VERSION   3004001 /* (1000000*MAJOR+1000*MINOR+PATCH)             */
+#define NCRYSTAL_VERSION_STR "3.4.1"
   NCRYSTAL_API int ncrystal_version(); /* returns NCRYSTAL_VERSION                  */
   NCRYSTAL_API const char * ncrystal_version_str(); /* returns NCRYSTAL_VERSION_STR */
 
+  /* Load raw NCMAT data into JSON structures. Must deallocate with call to        */
+  /* ncrystal_dealloc_string as usual. (WARNING: JSON is incomplete for now!!!!!)  */
+  NCRYSTAL_API char * ncrystal_ncmat2json( const char * textdataname );
+
+
   /*============================================================================== */
   /*============================================================================== */
   /*==                                                                          == */
@@ -527,6 +595,10 @@ extern "C" {
   /*============================================================================== */
   /*============================================================================== */
 
+  /*Obsolete function which now always returns 1.0. Packing factors are now        */
+  /*instead absorbed into the material densities:                                  */
+  NCRYSTAL_API double ncrystal_decodecfg_packfact( const char * cfgstr );
+
   /*Obsolete function which now just is an alias for ncrystal_clear_caches above:  */
   NCRYSTAL_API void ncrystal_clear_info_caches();
 
