@@ -2,7 +2,7 @@
 //                                                                            //
 //  This file is part of NCrystal (see https://mctools.github.io/ncrystal/)   //
 //                                                                            //
-//  Copyright 2015-2021 NCrystal developers                                   //
+//  Copyright 2015-2022 NCrystal developers                                   //
 //                                                                            //
 //  Licensed under the Apache License, Version 2.0 (the "License");           //
 //  you may not use this file except in compliance with the License.          //
@@ -525,35 +525,19 @@ NC::VectD::const_iterator NC::findClosestValInSortedVector(const NC::VectD& v, d
   return ncabs(*it-value) < ncabs(*std::prev(it)-value) ? it : std::prev(it);
 }
 
-std::pair<unsigned,unsigned> NC::detectSimpleRationalNumbers(double value)
-{
-  if (value<=0.0)
-    return { 0, ( value == 0.0 ? 1 : 0 ) };
-  if (value>=1.0) {
-    if (value==1.0)
-      return { 1, 1 };
-    double intpart;
-    if ( modf(value, &intpart) != 0.0 || intpart > 1.0e9 )
-      return {0,0};
-    return { static_cast<double>(intpart), 1 };
-  }
-  //ok, value is in (0,1).
-  static std::map<uint64_t,std::pair<unsigned,unsigned>> s_cache = []()
-  {
-    std::map<uint64_t,std::pair<unsigned,unsigned>> result;
-    for (unsigned b = 2; b <= 64; ++b) {
-      for (unsigned a = 1; a < b; ++a) {
-        uint64_t key = static_cast<uint64_t>((double(a)/b)*1e18);//fits in uint64_t since value of a/b is in (0,1)
-        auto it = result.find(key);
-        if (it==result.end())
-          result[key] = { a,b };//only insert if not there (because if a1/b1 = a2/b2, we prefer the smallest b-value).
-      }
-    }
-    return result;
-  }();
-  uint64_t key = static_cast<uint64_t>(value*1e18);
-  auto it = s_cache.find(key);
-  if ( it == s_cache.end() )
-    return {0,0};
-  return it->second;
-}
+static_assert( NC::ncconstexpr_ispow2( 1 ), "" );
+static_assert( NC::ncconstexpr_ispow2( 2 ), "" );
+static_assert( NC::ncconstexpr_ispow2( 4 ), "" );
+static_assert( NC::ncconstexpr_ispow2( 64 ), "" );
+static_assert( NC::ncconstexpr_ispow2( 1024 ), "" );
+static_assert( !NC::ncconstexpr_ispow2( 0 ), "" );
+static_assert( !NC::ncconstexpr_ispow2( 3 ), "" );
+static_assert( !NC::ncconstexpr_ispow2( 5 ), "" );
+static_assert( !NC::ncconstexpr_ispow2( 48 ), "" );
+static_assert( NC::ncconstexpr_roundupnextpow2(1) == 1, "" );
+static_assert( NC::ncconstexpr_roundupnextpow2(2) == 2, "" );
+static_assert( NC::ncconstexpr_roundupnextpow2(3) == 4, "" );
+static_assert( NC::ncconstexpr_roundupnextpow2(4) == 4, "" );
+static_assert( NC::ncconstexpr_roundupnextpow2(17) == 32, "" );
+static_assert( NC::ncconstexpr_roundupnextpow2(24) == 32, "" );
+static_assert( std::numeric_limits<double>::is_iec559, "NCrystal requires IEEE 754 floating point numbers" );
