@@ -2,7 +2,7 @@
 //                                                                            //
 //  This file is part of NCrystal (see https://mctools.github.io/ncrystal/)   //
 //                                                                            //
-//  Copyright 2015-2021 NCrystal developers                                   //
+//  Copyright 2015-2022 NCrystal developers                                   //
 //                                                                            //
 //  Licensed under the Apache License, Version 2.0 (the "License");           //
 //  you may not use this file except in compliance with the License.          //
@@ -23,6 +23,7 @@
 #include "NCrystal/internal/NCSABUtils.hh"
 #include "NCrystal/internal/NCFactoryUtils.hh"
 #include "NCrystal/internal/NCIter.hh"
+#include "NCrystal/internal/NCString.hh"
 #include <iostream>
 
 namespace NC = NCrystal;
@@ -33,7 +34,7 @@ struct NC::SAB::SABIntegrator::Impl : private NoCopyMove {
   Impl( shared_obj<const SABData>,
         const VectD* egrid,
         std::shared_ptr<const SABExtender> );
-  void doit(SABXSProvider *, SABSampler*);
+  void doit(SABXSProvider *, SABSampler*, Optional<std::string>*);
   double determineEMax( const double ) const;
   double determineEMin( const double ) const;
   void setupEnergyGrid();
@@ -46,6 +47,9 @@ struct NC::SAB::SABIntegrator::Impl : private NoCopyMove {
   //Data derived from m_data:
   std::shared_ptr<const SABSamplerAtE_Alg1::CommonCache> m_derivedData;
 
+  //Setting;
+  SABSampler::EGridMargin m_egridMargin;
+
   typedef std::unique_ptr<SABSamplerAtE> SamplerAtE_uptr;
   std::pair<SamplerAtE_uptr,double> analyseEnergyPoint(double ekin, bool doSampler ) const;
 
@@ -65,9 +69,9 @@ NS::SABIntegrator::SABIntegrator( shared_obj<const SABData> data,
 {
 }
 
-void NS::SABIntegrator::doit(SABXSProvider * out_xs, SABSampler* out_sampler)
+void NS::SABIntegrator::doit(SABXSProvider * out_xs, SABSampler* out_sampler, Optional<std::string>* json)
 {
-  m_impl->doit(out_xs,out_sampler);
+  m_impl->doit(out_xs,out_sampler,json);
 }
 
 NS::SABIntegrator::Impl::Impl( shared_obj<const SABData> data,
@@ -75,7 +79,8 @@ NS::SABIntegrator::Impl::Impl( shared_obj<const SABData> data,
                                std::shared_ptr<const SABExtender> sabextender )
   : m_data(std::move(data)),
     m_egrid((egrid&&!egrid->empty())?*egrid:VectD()),
-    m_extender(!sabextender?std::make_unique<SABFGExtender>(m_data->temperature(),m_data->elementMassAMU(),m_data->boundXS()):std::move(sabextender))
+    m_extender(!sabextender?std::make_unique<SABFGExtender>(m_data->temperature(),m_data->elementMassAMU(),m_data->boundXS()):std::move(sabextender)),
+    m_egridMargin{ 1.05 }
 {
 }
 
@@ -268,7 +273,7 @@ void NS::SABIntegrator::Impl::setupEnergyGrid()
 
 }
 
-void NS::SABIntegrator::Impl::doit(SABXSProvider * out_xs, SABSampler* out_sampler)
+void NS::SABIntegrator::Impl::doit(SABXSProvider * out_xs, SABSampler* out_sampler, Optional<std::string>* json)
 {
   nc_assert_always( out_xs || out_sampler );
   if ( !m_derivedData )
@@ -297,12 +302,35 @@ void NS::SABIntegrator::Impl::doit(SABXSProvider * out_xs, SABSampler* out_sampl
     out_sampler->setData( m_data->temperature(),
                           VectD(m_egrid.begin(),m_egrid.end()),
                           std::move(energyPointSamplers),
-                          m_extender, xsvals.back() );
+                          m_extender, xsvals.back(), m_egridMargin );
   if ( out_xs )
     out_xs->setData( VectD(m_egrid.begin(),m_egrid.end()),
                      std::move(xsvals),
                      m_extender );
 
+  if (json) {
+    std::ostringstream ss;
+    {
+      std::ostringstream tmp;
+      tmp << "nalpha="<<m_data->alphaGrid().size()<<";nbeta="<<m_data->betaGrid().size();
+      tmp << ";Emax="<<NeutronEnergy{m_egrid.back()};
+      tmp << ";T="<<m_data->temperature();
+      tmp << ";M="<<m_data->elementMassAMU();
+      tmp << ";sigma_free="<<m_data->boundXS().free(m_data->elementMassAMU());
+      streamJSONDictEntry( ss, "summarystr", tmp.str(), JSONDictPos::FIRST );
+    }
+    streamJSONDictEntry( ss, "Emax", m_egrid.back()  );
+    streamJSONDictEntry( ss, "Emin", m_egrid.front()  );
+    streamJSONDictEntry( ss, "negrid", m_egrid.size()  );
+    streamJSONDictEntry( ss, "T", m_data->temperature().dbl()  );
+    streamJSONDictEntry( ss, "M", m_data->elementMassAMU().dbl()  );
+    streamJSONDictEntry( ss, "sigma_bound", m_data->boundXS().dbl()  );
+    streamJSONDictEntry( ss, "sigma_free", m_data->boundXS().free(m_data->elementMassAMU()).dbl()  );
+    streamJSONDictEntry( ss, "nbeta", m_data->betaGrid().size()  );
+    streamJSONDictEntry( ss, "nalpha", m_data->alphaGrid().size(), JSONDictPos::LAST  );
+    *json = ss.str();
+  }
+
 }
 
 std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Impl::analyseEnergyPoint(double ekin, bool doSampler ) const
@@ -331,6 +359,7 @@ std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Im
   const double kT = m_data->temperature().kT();
   const double ekin_div_kT = ekin / kT;
   double beta_lower_limit = -ekin_div_kT;
+  bool starts_at_kinematic_endpoint = true;
   if (beta_lower_limit<betaGrid.front()) {
     //This can happen at high energies. Push up beta_lower_limit to juuuust
     //before the first grid point, thus keeping code simple without introducing
@@ -340,6 +369,7 @@ std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Im
                               betaGrid.front() - ncabs(betaGrid.front())*1e-13,
                               std::nexttoward(betaGrid.front(),beta_lower_limit) );
     nc_assert(beta_lower_limit<betaGrid.front());
+    starts_at_kinematic_endpoint = false;
   }
 
   //Figure out which sab alpha-ranges are involved, i.e. intersects the
@@ -361,6 +391,7 @@ std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Im
   if ( ibeta_low > 0 && beta_lower_limit < vectAt(betaGrid,ibeta_low-1) ) {
     //move up lower limit, there was apparently no kinematically accessible content further down.
     beta_lower_limit = vectAt(betaGrid,ibeta_low-1);
+    starts_at_kinematic_endpoint = false;
   }
 
   nc_assert( ibeta_low==0 || beta_lower_limit >= betaGrid.front() );
@@ -373,6 +404,7 @@ std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Im
   //We know (when beta_lower_limit=-E/kT, otherwise it is just something we
   //assume) that the cross sections take off from 0.0 at beta_lower_limit:
   PairDD prev_betaxs(beta_lower_limit,0.);
+
   std::vector<PairDD> betasampler_data;
   VectD betasampler_vals,betasampler_weights;
   std::vector<SABSamplerAtE_Alg1::AlphaSampleInfo> sampler_infos;
@@ -388,7 +420,12 @@ std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Im
 
   StableSum xs_total_stable;
 
+  bool next_bin_has_kinematic_endpoint = starts_at_kinematic_endpoint;
+
   for ( auto&& beta : enumerate(relevant_betaGrid) ) {
+    if ( beta.val == beta_lower_limit )
+      continue;//guard against lower limit falling exactly on a grid point
+
     nc_assert( beta.val >= beta_lower_limit );
     nc_assert( beta.val >= -ekin_div_kT );
     nc_assert( beta.idx < alpharanges.size() );
@@ -447,6 +484,24 @@ std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Im
       }
     }
 
+    if ( next_bin_has_kinematic_endpoint ) {
+      //The piece-wise-linear assumption is too crude in the first bin, we want to
+      //use a sqrt(x) shape instead. It just so happen that this increases the
+      //integral of the first bin by a factor of 4/3, which can be accommodated
+      //(faked!) in the piece-wise-linear machinary by moving the first beta value
+      //downwards by 1/3 of the first bin width. The only issue is that when
+      //sampling, we should resample values in the first bin according to the
+      //sqrt(x) shape, which is a piece of information we need to pass on to the
+      //SABSamplerAtE_Alg1 instance.
+      next_bin_has_kinematic_endpoint = false;
+      double db_real = (beta.val - prev_betaxs.first);
+      prev_betaxs.first -= db_real * (1.0 / 3.0);
+      if ( doSampler ) {
+        nc_assert( betasampler_vals.size() == 1 );
+        betasampler_vals.front() = prev_betaxs.first;
+      }
+    }
+
     //integrate trapezoidally from (beta,xs) = prev_betaxs
     xs_total_stable.add(0.5 * ( beta.val - prev_betaxs.first ) * ( xs_at_this_beta + prev_betaxs.second ));
 
@@ -478,6 +533,7 @@ std::pair<NS::SABIntegrator::Impl::SamplerAtE_uptr,double> NS::SABIntegrator::Im
                                                              std::move(betasampler_vals),
                                                              std::move(betasampler_weights),
                                                              std::move(sampler_infos),
-                                                             ibeta_low );
+                                                             ibeta_low,
+                                                             ( starts_at_kinematic_endpoint ? beta_lower_limit : 1.0 ) );
   return { std::move(up), xs_total };
 }
